﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace SharpConsole
{

    public delegate void TrickAction();

    public abstract class CircusAnimal
    {
        public int Legs { get; set; }
        public int LegsOnGround { get; set; }
        public int TricksDone { get; set; }
        public int IncentivesGiven { get; set; }

        protected string Sound;

        public void Speak()
        {
            Console.WriteLine(Sound);
            TricksDone += 1;
        }

        public static void WatchOthers()
        {
            Console.WriteLine("Watching is boring");
        }

        public void Stand()
        {
            if (LegsOnGround == 4)
                LegsOnGround -= 2;
            Console.WriteLine("Legs on Ground = {0}", LegsOnGround);
            TricksDone += 1;
        }

        public void DoTrick(TrickAction trick)
        {
            if (TricksDone <= IncentivesGiven)
                trick.Invoke();
            else
                Console.WriteLine("Need an incentive");

            this.OnRequestIncentive(null);
        }

        public void DoTrick(TrickAction trick, int incentivesGiven)
        {
            this.IncentivesGiven += incentivesGiven;
            DoTrick(trick);
        }

        public event EventHandler RequestIncentive;
        protected void OnRequestIncentive(EventArgs e)
        {
            EventHandler handler = RequestIncentive;
            if (handler != null)
                handler(this, e);
        }

        public List<TrickAction> TrickList = new List<TrickAction>();
        public void DoTricks()
        {
            foreach (TrickAction trick in TrickList)
                trick.Invoke();
        }

        private List<Predicate<CircusAnimal>> ValidationRules = new List<Predicate<CircusAnimal>>
            {(animal) => animal.LegsOnGround <= animal.Legs,
            (animal) => animal.Legs<=4};

        public bool IsValid()
        {
            foreach (var rule in ValidationRules)
            {
                if (!rule.Invoke(this))
                    return false;
            }
            return true;
        }
    }
}
