﻿using System;
using System.Collections.Generic;
//using System.Linq;
using System.Text;
using Delegates.Query;

namespace SharpConsole
{

    class Program
    {
        static void Main(string[] args)
        {
           
            //DoDelegateSamples();
            DoTrickList();
            Console.ReadLine();
        }

        private static List<CircusAnimal> GetAnimals()
        {
            List<CircusAnimal> animals = new List<CircusAnimal>()
                {new Lion(), new Tiger(), new Bear()};
            return animals;
        }
        
        private static void DoDelegateSamples()
        {
            List<CircusAnimal> animals = GetAnimals();

            foreach (CircusAnimal animal in animals)
            {
                Console.WriteLine(animal.GetType().Name);
                //Event handler delegate
                animal.RequestIncentive+=new EventHandler(GiveIncentive);

                //Instance delegate
                animal.DoTrick(animal.Stand);

                //Static delegate
                animal.DoTrick(CircusAnimal.WatchOthers);

                //Overloaded method
                animal.DoTrick(animal.Speak,2);

                //Named delegate
                animal.DoTrick(DelegateAction);

                //Anonymous delgate
                animal.DoTrick(delegate() { Console.WriteLine("Anonymous Method"); });

                
                //Lambda expression as a variable
                TrickAction LiftOneLeg = (() => animal.LegsOnGround -= 1);
                animal.DoTrick(LiftOneLeg);
                
                //Lambda expression inline
                animal.DoTrick(() => animal.LegsOnGround += 3);
                
                Console.WriteLine("Legs now on ground: {0}", animal.LegsOnGround);

                //Multi-line lambda
                animal.DoTrick(() =>
                {
                    animal.LegsOnGround = 0;
                    Console.WriteLine("I'm Jumping now!");
                    animal.LegsOnGround = 4;
                });

                Console.WriteLine("");
            }
        }

        private static void DoTrickList()
        {
            List<CircusAnimal> animals = GetAnimalsWithTrickList();

            foreach (CircusAnimal animal in animals)
            {
                Console.WriteLine(animal.GetType().Name);

                animal.DoTricks();
                Console.WriteLine("Legs now on ground: {0}", animal.LegsOnGround);
                Console.WriteLine("Animal valid: {0}", animal.IsValid());
                Console.WriteLine();

            }

            //Find invalid animals
            Console.WriteLine("Invalid: ");
           

            foreach (var animal in animals.WhereExpression(animal => !animal.IsValid()))
                Console.WriteLine(animal.ToString());
        }

        private static void DelegateAction()
        {
            Console.WriteLine("Performing an action as a named delegate");
        }

        private static void GiveIncentive(object sender, EventArgs e)
        {
            ((CircusAnimal)(sender)).IncentivesGiven += 1;
        }

        private static List<CircusAnimal> GetAnimalsWithTrickList()
        {
            List<CircusAnimal> animals = GetAnimals();

            animals[0].TrickList.Add(animals[0].Stand);
            animals[1].TrickList.Add(CircusAnimal.WatchOthers);
            animals[2].TrickList.Add(animals[2].Speak);
            animals[0].TrickList.Add(DelegateAction);
            animals[1].TrickList.Add(delegate() { Console.WriteLine("Anonymous Method"); });
            animals[2].TrickList.Add(() => animals[2].LegsOnGround += 3);
            return animals;
        }
    }
}
